"""@package utils
Output is a wrapper over print to provide coloring and verbose mode
"""

from __future__ import print_function

import os
import sys

from colorama import (
    init,
    Style,
)


class Scope:
    """holds the verbose flag and output status flag"""

    verbose_flag = False
    output_status_flag = True
    original_stdout = sys.stdout


init()
scope = Scope()


def set_verbose(status):
    """turns verbose flag on and off"""

    Scope.verbose_flag = status


def output_status(status):
    """turn output on and off"""

    if status:
        sys.stdout = scope.original_stdout
    else:
        scope.original_stdout = sys.stdout
        sys.stdout = open(os.devnull, 'w')


def verbose(string, newline, color=Style.RESET_ALL):
    """if verbose option is one, then only write on the output"""

    if Scope.verbose_flag is True:
        write(string, color)

        if newline is True:
            write('\n', color)


def write(string='', color=Style.RESET_ALL):
    """write a string with color specified. No new line"""

    print(color, end='')
    print(string, end='')
    print(Style.RESET_ALL, end='')


def writeln(string='', color=Style.RESET_ALL):
    """write a string with color specified. New line"""

    print(color, end='')
    print(string, end='')
    print(Style.RESET_ALL)



"""
Main Script that calls other scripts to make WQt work
"""

from __future__ import absolute_import

import argparse

from colorama import Fore

from wqt.command import creation, handle
from wqt.utils.output import writeln


def parse():
    """adds command line arguments and returns the options"""

    parser = argparse.ArgumentParser(description='WQt creates and builds Qt projects through CMake')

    parser.add_argument(
        'action',
        nargs='+',
        help='action to perform (create, update, build, clean, list-types, add-lib, rm-lib, list_qml, and show_qml')
    parser.add_argument(
        '--path',
        help='path where the project is or will be created'
    )
    parser.add_argument(
        '--generator',
        help='makefile generator to use for build (default: Unix Makefiles)',
        type=str)
    parser.add_argument(
        '--make',
        help='path to make binary',
        type=str)
    parser.add_argument(
        '--cmake',
        help='path to cmake binary',
        type=str)

    return parser.parse_args()


def provided(*args):
    """checks if given flags are specified during command line usage"""

    if any(flag is not None for flag in args):
        return True


def verify_qt_application(name):
    """verifies if the type of Qt application is supported"""

    if name != 'quick' and name != 'widgets' and name != 'console':
        writeln('Invalid Qt application specified', color=Fore.RED)
        quit(2)


def main():
    options = parse()

    path = None
    cmake = options.cmake
    make = options.make
    generator = options.generator

    if provided(options.path):
        path = str(options.path)

    # based on the action call scripts
    if 'create' in options.action:
        if len(options.action) < 2:
            writeln('Specify a type of Qt application to create', color=Fore.RED)
            quit(2)

        verify_qt_application(options.action[1])
        creation.create(path, options.action[1])
    elif 'update' in options.action:
        creation.update(path)
    elif 'build' in options.action:
        handle.build(path, generator, cmake, make)
    elif 'clean' in options.action:
        handle.clean(path)
    elif 'list-types' in options.action:
        handle.list_types()
    elif 'add-lib' in options.action:
        if len(options.action) < 2:
            writeln('Specify the name of the library to add', color=Fore.RED)
            quit(2)

        handle.add_lib(path, options.action[1])
    elif 'rm-lib' in options.action:
        if len(options.action) < 2:
            writeln('Specify the name of the library to remove', color=Fore.RED)
            quit(2)

        handle.rm_lib(path, options.action[1])
    elif 'run' in options.action:
        handle.run(path, generator, cmake, make)
    elif 'list-qml' in options.action:
        handle.list_qml(path)
    elif 'list-libs' in options.action:
        handle.list_libs(path)
    elif 'preview-qml' in options.action:
        if len(options.action) < 2:
            writeln('Specify the name of the qml file to preview', color=Fore.RED)
            quit(2)

        handle.preview_qml(path, options.action[1])


if __name__ == '__main__':
    main()



"""@package command
Handle building, listing, and showing WQt projects
"""

import json
import os
import shutil
import subprocess
from collections import OrderedDict

from colorama import Fore

from wqt.utils.finder import (
    get_cmake_program,
    get_generator_for,
    get_make_program,
    get_qmlscene_program,
    get_qmlviewer_program
)
from wqt.utils.helper import (
    get_files,
    get_dirs,
    get_valid_path,
    get_platform,
    get_qt_application
)
from wqt.utils.output import writeln, write
from . import creation


def build(path, generator=None, make=None, cmake=None):
    """build WQt project"""

    path = get_valid_path(path)

    writeln('WQt project build started', Fore.YELLOW)
    write('Verifying project structure - ', Fore.CYAN)

    application = get_qt_application(path)

    valid = False
    if os.path.exists(path + '/wqt') and os.path.exists(path + '/src') and os.path.exists(path + '/src/app') \
            and os.path.exists(path + '/res'):
        if application == 'widgets' and os.path.exists(path + '/res/ui'):
            valid = True
        elif application == 'quick' and os.path.exists(path + '/res/qml'):
            valid = True
        elif application == 'console':
            valid = True

    if valid:
        writeln('done')
    else:
        writeln('\nProject structure invalid, update the project', color=Fore.RED)
        quit(2)

    # check if build folder exists and if not make one
    if not os.path.exists(path + '/wqt/build'):
        os.mkdir(path + '/wqt/build')

    os.chdir(path + '/wqt/build')

    cmake_program = cmake or get_cmake_program()
    make_program = make or get_make_program()

    write('Verifying cmake and make installs - ', Fore.CYAN)

    # check if cmake is in environment paths (unix/linux based systems)
    if not cmake_program:
        writeln('\ncmake does not exist, please install it or make sure it is in your environment PATH', Fore.RED)
        quit(2)

    # check if make is in environment paths (unix/linux based systems)
    if not make_program:
        writeln('\nmake does not exist, please install it or make sure it is in your environment PATH', Fore.RED)
        quit(2)

    writeln('done')

    if generator is None:
        generator = get_generator_for(make_program)

    writeln('Running the build using cmake and ' + str(generator), Fore.CYAN)
    cmake_code = subprocess.call(['cmake', '-G', str(generator), '../..'])

    if cmake_code != 0:
        writeln('Project build unsuccessful, cmake exited with error code ' + str(cmake_code), Fore.RED)
        quit(2)

    make_code = subprocess.call([make_program])

    if make_code != 0:
        writeln('Project build unsuccessful, make exited with error code ' + str(make_code), Fore.RED)
        quit(2)

    writeln('Project successfully built', Fore.YELLOW)

    pass


def clean(path):
    """clean WQt project's build folder"""

    path = get_valid_path(path)

    # confirm if bin folder/path exists
    if not os.path.exists(path + '/wqt/build'):
        writeln('No build files to clean', color=Fore.YELLOW)
        quit(2)

    # check if the current build files are for the current board
    # clean and build if boards are different
    try:
        write('Cleaning build files - ', Fore.CYAN)

        for folder in get_dirs(path + '/wqt/build'):
            shutil.rmtree(folder)

        for file in get_files(path + '/wqt/build'):
            os.unlink(file)
    except IOError:
        writeln('Error while cleaning build files', Fore.CYAN)

    shutil.rmtree(path + '/wqt/build')

    writeln('done')
    writeln('Project build files cleaned', color=Fore.YELLOW)


def list_types():
    """lists WQt application types supported"""

    writeln('Application types supported are: ', color=Fore.YELLOW)
    writeln('widgets', color=Fore.CYAN)
    writeln('quick', color=Fore.CYAN)
    writeln('console', color=Fore.CYAN)
    writeln('#################################')
    write('Example usage: ', color=Fore.YELLOW)
    writeln('wqt create widgets', color=Fore.YELLOW)


def add_lib(path, name):
    """add Qt library to the project"""

    path = get_valid_path(path)

    write('Adding library named ' + name + ' - ', color=Fore.CYAN)

    # load config file as json
    with open(path + '/config.json') as f:
        config_data = json.load(f, object_pairs_hook=OrderedDict)

    if not str(name) in config_data['libraries-qt']:
        config_data['libraries-qt'].append(str(name))
    else:
        writeln('\nLibrary already exist, libraries are: ' + ' '.join(config_data['libraries-qt']),
                color=Fore.YELLOW)
        quit(2)

    # write the project name to config file
    with open(path + '/config.json', 'w') as f:
        json.dump(config_data, f, indent=2)

    writeln('done')
    writeln('Libraries left are: ' + ' '.join(config_data['libraries-qt']), color=Fore.YELLOW)
    writeln('#################################################################################')
    writeln('Updating CMake files: ', color=Fore.CYAN)
    creation.update(path)


def rm_lib(path, name):
    """remove Qt library from the project"""

    path = get_valid_path(path)

    write('Removing library named ' + name + ' - ', color=Fore.CYAN)

    # load config file as json
    with open(path + '/config.json') as f:
        config_data = json.load(f, object_pairs_hook=OrderedDict)

    if str(name) in config_data['libraries-qt']:
        config_data['libraries-qt'].remove(str(name))
    else:
        writeln('\nNo such library to remove, libraries are: ' + ' '.join(config_data['libraries-qt']),
                color=Fore.YELLOW)
        quit(2)

    # write the project name to config file
    with open(path + '/config.json', 'w') as f:
        json.dump(config_data, f, indent=2)

    writeln('done')
    writeln('Libraries left are: ' + ' '.join(config_data['libraries-qt']), color=Fore.YELLOW)
    writeln('#################################################################################')
    writeln('Updating CMake files: ', color=Fore.CYAN)
    creation.update(path)


def list_libs(path):
    """list Qt library used for the project"""

    path = get_valid_path(path)

    writeln('Libraries used in the projects:', color=Fore.YELLOW)

    # load config file as json
    with open(path + '/config.json') as f:
        config_data = json.load(f, object_pairs_hook=OrderedDict)

    for lib in config_data['libraries-qt']:
        writeln(lib, color=Fore.CYAN)


def run(path, generator=None, cmake=None, make=None):
    """open the binary executable file"""

    path = get_valid_path(path)

    build(path, generator, cmake, make)

    with open(path + '/config.json') as f:
        config_data = json.load(f)

    executable = config_data['name-project']

    if get_platform() == 'OS X':
        subprocess.call(['open', path + '/bin/' + executable + '.app'])
    elif get_platform() == 'Windows':
        subprocess.call([path + '/bin/' + executable + '.exe'])
    else:
        subprocess.call([path + '/bin/' + executable])


def list_qml(path):
    """list qml files in the project"""

    path = get_valid_path(path)

    if get_qt_application(path) == 'quick':
        writeln('This project is not a Qt Quick project so no qml files', color=Fore.YELLOW)
        quit(2)

    writeln('Qml files for this project: ', color=Fore.YELLOW)
    files = get_files(path + '/res/qml')

    for file in files:
        if os.path.splitext(file)[1] == '.qml':
            writeln(os.path.basename(file), color=Fore.CYAN)


def preview_qml(path, name):
    """preview qml file"""

    path = get_valid_path(path)

    if not os.path.exists(path + '/wqt/quick'):
        writeln('This project is not a Qt Quick project so no qml preview :(', color=Fore.YELLOW)
        quit(2)

    qml_path = ''

    if os.path.exists(path + '/res/qml/' + name):
        qml_path = path + '/res/qml/' + name
    elif os.path.exists(path + '/res/qml/' + name + '.qml'):
        qml_path = path + '/res/qml/' + name + '.qml'

    if get_qmlscene_program():
        subprocess.call(['qmlscene', qml_path])
    elif get_qmlviewer_program():
        subprocess.call(['qmlviewer', qml_path])
    else:
        writeln('No Qml viewer program install please install qmlscene or qmlviewer', color=Fore.RED)
        quit(2)



"""@package utils
Parses and completes the cmake templates
"""

from wqt.utils import helper


def parse_update(tpl_path, project_data):
    """reads the cmake template file and completes it using project data"""
    with open(tpl_path) as f:
        tpl_str = f.readlines()

    new_str = ''
    index = 0
    while index < len(tpl_str):
        curr_line = tpl_str[index]
        new_str += helper.fill_template(curr_line, project_data)
        index += 1

    # link libraries
    libraries = project_data['libraries-qt']
    link_str = ''

    for library in libraries:
        link_str += 'PUBLIC Qt5::'
        link_str += library
        link_str += ' '

    link_str = link_str.strip(' ')
    new_str = new_str.replace('{{link-libraries}}', link_str)

    with open(tpl_path, 'w') as f:
        f.write(new_str)



import functools


# @memoized decorator taken from
# https://github.com/apache/incubator-superset/
class _memoized(object):  # noqa
    """Decorator that caches a function's return value each time it is called
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.
    Define ``watch`` as a tuple of attribute names if this Decorator
    should account for instance variable changes.
    """

    def __init__(self, func, watch=()):
        self.func = func
        self.cache = {}
        self.is_method = False
        self.watch = watch

    def __call__(self, *args, **kwargs):
        key = [args, frozenset(kwargs.items())]
        if self.is_method:
            key.append(tuple([getattr(args[0], v, None) for v in self.watch]))
        key = tuple(key)
        if key in self.cache:
            return self.cache[key]
        try:
            value = self.func(*args, **kwargs)
            self.cache[key] = value
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            # Better to not cache than to blow up entirely.
            return self.func(*args, **kwargs)

    def __repr__(self):
        """Return the function's docstring."""
        return self.func.__doc__

    def __get__(self, obj, objtype):
        if not self.is_method:
            self.is_method = True
        """Support instance methods."""
        return functools.partial(self.__call__, obj)


def memoized(func=None, watch=None):
    if func:
        return _memoized(func)
    else:
        def wrapper(f):
            return _memoized(f, watch)

        return wrapper



"""@package utils
Helper functions to be used through the tool
"""

import os
import sys
import shutil
from os.path import abspath, dirname
from colorama import Fore
from .output import writeln

from six import string_types


def get_platform():
    """Returns the operating system"""

    platforms = {
        'linux1': 'Linux',
        'linux2': 'Linux',
        'darwin': 'OS X',
        'win32': 'Windows'
    }
    if sys.platform not in platforms:
        return sys.platform

    return platforms[sys.platform]


def verify_path(path):
    """check if the project path is correct"""

    if not os.path.exists(path) or not os.path.isdir(path):
        writeln('Path specified for project creation does not exist or is not a directory', color=Fore.RED)
        quit(2)


def get_valid_path(path):
    """gets the project path based on if it is provided or not"""

    if path is None:
        path = get_working_directory()
    else:
        path = linux_path(os.path.abspath(path))

    verify_path(path)

    return path


def get_qt_application(path):
    """returns the type of qt application"""

    if os.path.exists(path + '/wqt/console'):
        application = 'console'
    elif os.path.exists(path + '/wqt/quick'):
        application = 'quick'
    elif os.path.exists(path + '/wqt/widgets'):
        application = 'widgets'
    else:
        application = None

    return application


def quote_join(values):
    """Join a set of strings, presumed to be file paths, surrounded by quotes, for CMake"""

    surrounded = []
    for value in values:
        surrounded.append('\"' + value + '\"')
    return ' '.join(surrounded)


def linux_path(path):
    """Converts Windows style path to linux style path"""

    return os.path.abspath(path).replace('\\', '/')


def get_wqt_path():
    """returns the absolute path of wcosa"""

    wcosa_path = dirname(abspath(__file__))
    return linux_path(dirname(dirname(wcosa_path)))


def get_cmake_path():
    """returns the absolute path of cosa"""

    return linux_path(get_wqt_path() + '/toolchain/cmake')


def get_working_directory():
    """get path from where the script is called"""

    return linux_path(os.path.abspath(os.getcwd()))


def fill_template(string, data):
    """Fills the template based on the data provided"""

    string = string.replace('\\n', '\n').replace('\\t', '\t')

    for key in data:
        value = str(data[key])

        if isinstance(data[key], list):
            value = ' '.join(data[key])
        elif isinstance(data[key], string_types):
            value = data[key]

        if '{{' + key + '}}' in string:
            return string.replace('{{' + key + '}}', value)

    return string


def create_folder(path, override=False):
    """Creates a folder at the given path"""

    if override:
        if os.path.exists(path):
            shutil.rmtree(path)
        os.mkdir(path)
    elif not os.path.exists(path):
        os.mkdir(path)


def get_files_recursively(directory, extensions=None):
    """gathers a list of all the files with the extensions recursively in a directory"""

    arr = []
    for root, dirs, files in os.walk(directory):
        path = root.split(os.sep)
        for file in files:
            ext = os.path.splitext(file)[1]

            if extensions is not None and ext in extensions:
                arr.append(linux_path('/'.join(path) + '/' + file))
            elif extensions is None:
                arr.append(linux_path('/'.join(path) + '/' + file))

    return arr


def get_files(path, extensions=None):
    """gathers a list of all the files with the extensions in a directory"""

    arr = []
    all_files = os.listdir(path)

    for file in all_files:
        if not os.path.isdir(path + '/' + file) and extensions is not None and os.path.splitext(file)[1] in extensions:
            arr.append(linux_path(path + '/' + file))
        elif not os.path.isdir(path + '/' + file) and extensions is None:
            arr.append(linux_path(path + '/' + file))

    return arr


def get_dirs_recursively(path):
    """gathers a list of all the subdirectories recursively inside the path"""

    arr = []
    for root, dirs, files in os.walk(path):
        curr_path = '/'.join(root.split(os.sep))

        if curr_path != os.path.basename(path):
            arr.append(linux_path(curr_path))

    return arr


def get_dirs(path):
    """gathers a list of all the subdirectories inside the path"""

    arr = []
    for file in os.listdir(path):
        if os.path.isdir(path + '/' + file):
            arr.append(linux_path(path + '/' + file))

    return arr


def get_dirnames(path):
    """gathers a list of all the names of subdirectories inside the path"""

    arr = []
    for file in os.listdir(path):
        if os.path.isdir(path + '/' + file):
            arr.append(os.path.basename(linux_path(path + '/' + file)))

    return arr



#!/usr/bin/env python

import os

from setuptools import find_packages, setup

install_requires = [
    'colorama',
    'six'
]


def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths


setup(
    name='WQt',
    version='1.0.7',
    description='Create, Build, and Run Qt Projects',
    author='Deep Dhillon',
    author_email='deep@deepdhillon.ca',
    url='https://github.com/dhillondeep/wqt',  # use the URL to the github repo
    long_description=open('doc/README.rst').read(),
    license='MIT',
    packages=find_packages(),
    install_requires=install_requires,
    package_data={
        '': package_files('toolchain') + package_files('templates') + package_files('doc'),
        'wqt': ['*.json'],
    },
    entry_points={
        'console_scripts': [
            'wqt = wqt.wqt:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta'
    ],
    keywords=[
        'qt', 'c++', 'cmake', 'make', 'tool'
    ],
)



import os

from .memoize import memoized

make_programs = [
    'make',
    'make.exe',
    'mingw32-make',
    'mingw32-make.exe',
]

cmake_programs = [
    'cmake',
    'cmake.exe',
]

qmlscene_program = [
    'qmlscene',
    'qmlscene.exe'
]

qmlviewer_program = [
    'qmlviewer',
    'qmlviewer.exe'
]


@memoized
def cmd_exists(cmd):
    """checks if the program exists in the PATH"""

    return any(
        os.access(os.path.join(path, cmd), os.X_OK)
        for path in os.environ['PATH'].split(os.pathsep)
    )


def get_program(programs):
    for program in programs:
        if cmd_exists(program):
            return program
    return None


@memoized
def get_generator_for(make_program):
    if 'mingw' in make_program:
        return 'MinGW Makefiles'
    return 'Unix Makefiles'


@memoized
def get_make_program():
    return get_program(make_programs)


@memoized
def get_cmake_program():
    return get_program(cmake_programs)


@memoized
def get_qmlscene_program():
    return get_program(qmlscene_program)


@memoized
def get_qmlviewer_program():
    return get_program(qmlviewer_program)



"""@package command
Handle creation, and update of WQt projects
"""

import json
import os
from collections import OrderedDict
from distutils.dir_util import copy_tree
from shutil import copyfile

from colorama import Fore

from wqt.utils import cmake
from wqt.utils.helper import (
    create_folder,
    linux_path,
    get_wqt_path,
    get_files,
    get_dirs,
    get_valid_path,
    get_platform,
    get_qt_application
)
from wqt.utils.output import (
    writeln,
    write
)


def create_folders(project_path, application, override=False):
    """Creates required folders (src, bin, lib and res) in the project directory"""

    create_folder(project_path + '/src', override)
    create_folder(project_path + '/src/app', override)
    create_folder(project_path + '/lib', override)
    create_folder(project_path + '/wqt', override)

    if get_platform() == 'OS X':
        create_folder(project_path + "/res")
        create_folder(project_path + "/res/icons")

    if application == 'quick':
        if not os.path.exists(project_path + '/res'):
            create_folder(project_path + '/res', override)
        create_folder(project_path + '/res/qml', override)
    elif application == 'widgets':
        if not os.path.exists(project_path + '/res'):
            create_folder(project_path + '/res', override)
        create_folder(project_path + '/res/ui', override)


def verify_path(path):
    """check if the project path is correct"""

    if not os.path.exists(path) or not os.path.isdir(path):
        writeln('Path specified for project creation does not exist or is not a directory', color=Fore.RED)
        quit(2)


def copy_cmake_template_file(template_path, copy_path):
    """copy cmake template file to the project created"""

    if get_platform() == 'OS X':
        copyfile(template_path + '/cmake/CMakeLists-Apple.txt.tpl', copy_path + '/CMakeLists.txt')
    else:
        copyfile(template_path + '/cmake/CMakeLists.txt.tpl', copy_path + '/CMakeLists.txt')


def copy_config_template_file(template_path, copy_path):
    """copy config template file to the project created"""

    if get_platform() == 'OS X':
        copyfile(template_path + '/config/config-apple.json.tpl', copy_path + '/config.json')
    else:
        copyfile(template_path + '/config/config.json.tpl', copy_path + '/config.json')


def copy_application_files(application, templates_path, path):
    """copy Qt default files based on the application type"""

    if application == 'widgets':
        copy_widgets_files(templates_path, path)
    elif application == 'quick':
        copy_quick_files(templates_path, path)
    elif application == 'console':
        copy_console_files(templates_path, path)


def copy_widgets_files(templates_path, path):
    """copy widgets type files"""

    copyfile(templates_path + '/widgets/widgets', path + '/wqt/widgets')
    copyfile(templates_path + '/widgets/main.cpp', path + '/src/app/main.cpp')
    copyfile(templates_path + '/widgets/mainwindow.cpp', path + '/src/app/mainwindow.cpp')
    copyfile(templates_path + '/widgets/mainwindow.h', path + '/src/app/mainwindow.h')
    copyfile(templates_path + '/widgets/mainwindow.ui', path + '/res/ui/mainwindow.ui')


def copy_quick_files(templates_path, path):
    """copy quick type files"""

    copyfile(templates_path + '/quick/quick', path + '/wqt/quick')
    copyfile(templates_path + '/quick/main.cpp', path + '/src/app/main.cpp')
    copyfile(templates_path + '/quick/main.qml', path + '/res/qml/main.qml')


def copy_console_files(templates_path, path):
    """copy console type files"""

    copyfile(templates_path + '/console/console', path + '/wqt/console')
    copyfile(templates_path + '/console/main.cpp', path + '/src/app/main.cpp')


def create(path, application):
    """Creates WQt project from scratch"""

    path = get_valid_path(path)
    project_name = os.path.basename(path)

    writeln('Creating Qt ' + application + ' project', color=Fore.YELLOW)

    templates_path = linux_path(get_wqt_path() + '/templates')
    toolchain_path = linux_path(get_wqt_path() + '/toolchain')

    write('Creating Qt work environment - ', color=Fore.CYAN)

    # check if there are files in the folder
    if get_dirs(path) or get_files(path):
        if os.path.exists(path + '/src') or os.path.exists(path + '/lib') or os.path.exists(path + '/res') or \
                os.path.exists(path + '/cmake'):
            writeln('\nThere is already a src/lib/res/cmake folder in this directory. Use wqt update instead',
                    color=Fore.RED)
            quit(2)

    create_folders(path, application, True)
    writeln('done')

    write('Copying required files - ', color=Fore.CYAN)
    copy_tree(toolchain_path + '/cmake', path + '/wqt/cmake')
    copy_tree(toolchain_path + '/helper', path + '/wqt/helper')
    copy_cmake_template_file(templates_path, path)
    copy_config_template_file(templates_path, path)
    copyfile(templates_path + '/gitignore.tpl', path + '/.gitignore')
    copyfile(templates_path + '/readme/app-README.md.tpl', path + '/src/app/README.md')
    copyfile(templates_path + '/readme/lib-README.md.tpl', path + '/lib/README.md')

    if get_platform() == 'OS X':
        copyfile(templates_path + '/icon.icns', path + '/res/icons/icon.icns')

    copy_application_files(application, templates_path, path)

    writeln('done')

    write('Applying configurations - ', color=Fore.CYAN)

    # load config file as json
    with open(path + '/config.json') as f:
        config_data = json.load(f, object_pairs_hook=OrderedDict)

    config_data['name-project'] = project_name

    if application == 'quick':
        config_data['libraries-qt'].append(str('Gui'))
        config_data['libraries-qt'].append(str('Qml'))
    elif application == 'widgets':
        config_data['libraries-qt'].append(str('Gui'))
        config_data['libraries-qt'].append(str('Widgets'))

    # write the project name to config file
    with open(path + '/config.json', 'w') as f:
        json.dump(config_data, f, indent=2)

    # fill the template
    cmake.parse_update(path + "/CMakeLists.txt", config_data)
    writeln('done')
    writeln('Qt project created', color=Fore.YELLOW)


def update(path):
    """Updates existing WQt project"""

    path = get_valid_path(path)

    project_name = os.path.basename(path)
    application = get_qt_application(path)

    writeln('Updating Qt ' + application + ' project', color=Fore.YELLOW)

    templates_path = linux_path(get_wqt_path() + '/templates')
    toolchain_path = linux_path(get_wqt_path() + '/toolchain')

    write('Updating Qt work environment - ', color=Fore.CYAN)

    create_folders(path, application, False)
    writeln('done')

    write('Copying required files - ', color=Fore.CYAN)
    copy_tree(toolchain_path + '/cmake', path + '/wqt/cmake')
    copy_tree(toolchain_path + '/helper', path + '/wqt/helper')
    copy_cmake_template_file(templates_path, path)

    if not os.path.exists(path + '/config.json'):
        copy_config_template_file(templates_path, path)

    if not os.path.exists(path + '/gitignore'):
        copyfile(templates_path + '/gitignore.tpl', path + '/.gitignore')

    if not os.path.exists(path + '/src/app/README.mod'):
        copyfile(templates_path + '/readme/app-README.md.tpl', path + '/src/app/README.md')

    if not os.path.exists(path + '/lib/README.md'):
        copyfile(templates_path + '/readme/lib-README.md.tpl', path + '/lib/README.md')

    if get_platform() == 'OS X' and not os.path.exists(path + '/res/icons/icon.icns'):
        copyfile(templates_path + '/icon.icns', path + '/res/icons/icon.icns')

    writeln('done')

    write('Applying updated configurations - ', color=Fore.CYAN)

    # load config file as json
    with open(path + '/config.json') as f:
        config_data = json.load(f, object_pairs_hook=OrderedDict)

    config_data['name-project'] = project_name

    # write the project name to config file
    with open(path + '/config.json', 'w') as f:
        json.dump(config_data, f, indent=2)

    # fill the template
    cmake.parse_update(path + "/CMakeLists.txt", config_data)
    writeln('done')
    writeln('Qt project updated', color=Fore.YELLOW)
